#!/usr/bin/python

"""This file provides functions to runs the experiments with following configurations:
using following parameters:
logPath: path of the log file
revisionToUse: hadoop version; [0,2,4,8]
leakConfigName: name of leak
leakStrength: strength of the leak [1, 10, 100, 1000]
leakIndex: tye of artificial leak [1,2,3,4]
numOfIteration: 1
resultDirName: result directory
profiling: Profiling or noProfiling
isInstrumented: instrumented or not
passId: Pass id of the new experiment (result directory name)
"""


from experiment import *
from toolsAgingScripts import *
from constants import *


def q2_with_leak(logPath, revisionToUse, leakConfigName, leakIndex, leakStrength,
                 numOfIteration, resultDirName, profiling, isInstrumented):
    """runs the experiments designed to answer Q2:
    using different hadoop versions, with different leaks and instrumented run
    @param leakIndex: 1,2,3,4 (leak number)
    @param leakStrength: 1,10,100,1000
    """
    for version in revisionToUse:
        for index in leakIndex:
            for strength in leakStrength:
                leak_property = index * strength
                leak_property_str = str(leak_property)
                for iteration in numOfIteration:
                    logFile = createLogFile(logPath, version, leakConfigName, iteration)
                    sys.stdout = open(logFile, 'w')
                    experiment(version, leakConfigName + leak_property_str,
                               iteration, resultDirName, profiling, isInstrumented)
    return


def q2_leak_5(logPath, revisionToUse, leakConfigName, numOfIteration,
              resultDirName, profiling, isInstrumented):
    """runs the experiments designed to answer Q2:
    using different hadoop versions, with real leak (leak #5) and instrumented run
    """
    leakConfigName = 'leak5'
    for version in revisionToUse:
        for iteration in numOfIteration:
            logFile = createLogFile(logPath, version, leakConfigName, iteration)
            sys.stdout = open(logFile, 'w')
            experiment(version, leakConfigName, iteration, resultDirName, profiling, isInstrumented)
    return


def q2_no_leak(logPath, revisionToUse, leakConfigName, numOfIteration,
               resultDirName, profiling, isInstrumented):
    """runs the experiments designed to answer Q2:
    using different hadoop versions, no leak and instrumented run
    """
    leakConfigName = 'noLeak'
    for version in revisionToUse:
        for iteration in numOfIteration:
            logFile = createLogFile(logPath, version, leakConfigName, iteration)
            sys.stdout = open(logFile, 'w')
            experiment(version, leakConfigName, iteration, resultDirName, profiling, isInstrumented)
    return


def main(passId):
    """
    main function for running three q2 functions
    @param passId: Pass id of the new experiment
    """
    logPath = BASE_DIR
    resultDirName = 'Q2/' + passId
    revisionToUse = ['0', '2', '4', '8']
    numOfIteration = ['1']
    leakConfigName = 'leak'
    leakIndex = [1, 2, 3, 4]
    leakStrength = [1, 10, 100, 1000]
    profiling = 'noProfiling'
    isInstrumented = 'instrumented'

    q2_with_leak(logPath, revisionToUse, leakConfigName, leakIndex, leakStrength,
                 numOfIteration, resultDirName, profiling, isInstrumented)
    q2_no_leak(logPath, revisionToUse, leakConfigName, numOfIteration,
               resultDirName, profiling, isInstrumented)
    q2_leak_5(logPath, revisionToUse, leakConfigName, numOfIteration,
              resultDirName, profiling, isInstrumented)


if __name__ == "__main__":
    # check parameter amount
    if total_cmd_args() < 1:
        print ("Missing arguments: <passId>")
        exit()
    file_name, rest_args = sys.argv[0], str(sys.argv[1])
    passId = rest_args
    main(passId)
