"""Description: This file provides functions to setup
HDFS and Yarn for running experiments using following parameters :
revisionToUse: hadoop revision to use; e.g. 0,2,4,8
leakConfigName: name of the leak configuration to use (w/o ".properties")
isInstrumented: [instrumented]
"""

import ConfigParser
from constants import *
from toolsAgingScripts import *
import os


# primary function of this file
def start(revisionToUse, leakConfigName, isInstrumented):
    # load the leak properties and apply them to hadoop YARN
    # read leakProperties config file
    listLeakValue = []
    config = ConfigParser.RawConfigParser()
    config.read('leakProperties.cfg')
    for leakType in listLeakTypes:
        listLeakValue.append(config.get(leakConfigName, leakType))

    #add two lists in formatted string
    yarnOptions = ' '.join('%s=%s' % t for t in zip(listLeakVariable, listLeakValue))

    # setup env variables
    HADOOP_YARN_HOME = os.path.join(BASE_DIR + revisionToUse)
    HADOOP_MAPRED_HOME = os.path.join(BASE_DIR + revisionToUse)
    HADOOP_COMMON_HOME = os.path.join(HADOOP_DIR + revisionToUse)
    HADOOP_HDFS_HOME = os.path.join(HADOOP_DIR + revisionToUse)
    HADOOP_CONF_DIR = os.path.join(os.path.join(HADOOP_DIR + revisionToUse), 'etc/hadoop')
    YARN_CONF_DIR = os.path.join(os.path.join(HADOOP_DIR + revisionToUse), 'etc/hadoop')
    HADOOP_REVISION_DIR = os.path.join(HADOOP_DIR + revisionToUse)
    START_LOG = os.path.join(HADOOP_REVISION_DIR, 'start.log')
    HADOOP_REVISION_BIN_DIR = os.path.join(os.path.join(HADOOP_DIR + revisionToUse), 'bin')
    HADOOP_REVISION_SBIN_DIR = os.path.join(os.path.join(HADOOP_DIR + revisionToUse), 'sbin')
    HADOOP_SHELL = os.path.join(HADOOP_REVISION_BIN_DIR, 'hadoop')
    START_DFS_SHELL = os.path.join(HADOOP_REVISION_SBIN_DIR, 'start-dfs.sh')
    YARN_DAEMON_SHELL = os.path.join(HADOOP_REVISION_SBIN_DIR, 'yarn-daemon.sh')

    os.environ["CLASSPATH"] = ("%s:%s:%s:%s:%s:%s:%s" %
                               (JAVA_HOME, HADOOP_YARN_HOME,
                                HADOOP_MAPRED_HOME, HADOOP_COMMON_HOME,
                                HADOOP_HDFS_HOME, HADOOP_CONF_DIR, YARN_CONF_DIR))

    # setup HDFS
    print "starting HDFS ..."
    listCmdArgs = ['namenode', '-format', '-force']
    run_process(HADOOP_SHELL, listCmdArgs, HADOOP_REVISION_BIN_DIR, START_LOG)

    os.chdir(HADOOP_REVISION_SBIN_DIR)
    listCmdArgs = []
    run_process(START_DFS_SHELL, listCmdArgs, HADOOP_REVISION_SBIN_DIR, START_LOG)

    # setup YARN daemons
    print "setting up YARN ..."

    yarnOptionsOld = yarnOptions
    javaAgentPath = (' -javaagent:/home/felix/workspace/SoftwareAgingRCA/dist/'
                     'LiveObjectMonitor.jar=/home/felix/hadoop/%s/src/java,' % revisionToUse)
    resourceOption = javaAgentPath + 'ResourceManager'
    nodeOption = javaAgentPath + 'NodeManager'

    if 'v' + isInstrumented == "vinstrumented":
        os.environ["YARN_OPTS"] = ("%s%s" % (yarnOptionsOld, resourceOption))

    listCmdArgs = ['start', 'resourcemanager']
    run_process(YARN_DAEMON_SHELL, listCmdArgs, HADOOP_REVISION_SBIN_DIR, START_LOG)

    if 'v' + isInstrumented == "vinstrumented":
        os.environ["YARN_OPTS"] = ("%s%s" % (yarnOptionsOld, nodeOption))

    listCmdArgs = ['start', 'nodemanager']
    run_process(YARN_DAEMON_SHELL, listCmdArgs, HADOOP_REVISION_DIR, START_LOG)
    listCmdArgs = []
    run_process('jps', listCmdArgs, None, START_LOG)
    print "...finished..."

if __name__ == "__main__":
    # check count of cmd line params
    if total_cmd_args() == 0:
        print "Missing arguments!; usage is start.py <revision> <leakConf> | [instrumented]"
        exit()
    fileName, restArgs = sys.argv[0], sys.argv[1:]
    revisionToUse, leakConfigName, isInstrumented = restArgs

    start(revisionToUse, leakConfigName, isInstrumented)
