#!/usr/bin/python

"""This file provides functions to start Hadoop environment for running new experiments"""


from start import start
from constants import *
from toolsAgingScripts import *


# Primary function of this file
def startup(revisionToUse, leakConfigName, resultDirName, numMinsToSleep, isInstrumented=""):
    """Prepares the environment for a new experiment, including start of Hadoop
    @rtype : object
    @param revisionToUse: hadoop revision to use, e.g. 0,2,4,8
    @param leakConfigName: name of the leak configuration to use (w/o ".properties")
    @param resultDirName: the dir to put the result files
    @param numMinsToSleep: number of minutes to sleep after Hadoop startup
    @param isInstrumented: [instrumented] if it is instrumented or not
    """
    print
    print "#################################"

    # open experiment configuration file and write to it
    expConfigFile = resultDirName + '/exp.conf'
    f = open(expConfigFile, 'a')
    f.write("Environment start date: % s" % current_time())
    f.write("\n")
    f.close()
    # call start hadoop function;
    start(revisionToUse, leakConfigName, isInstrumented)

    # sleep for #mins_to_sleep
    sleep(numMinsToSleep)

    print "#################################"
    print
    print ("%s: ... started the environment ..." % current_time())


if __name__ == "__main__":
    # check count of cmd line params
    if total_cmd_args() < 5:
        print "Missing arguments; usage is startup.sh <revision> <leakConf> <result-dir> <sleep> [instrumented]"
        exit()
    fileName, restArgs = sys.argv[0], sys.argv[1:]
    if total_cmd_args() == 5:
        revisionToUse, leakConfigName, resultDirName, numMinsToSleep = restArgs
        isInstrumented = ""
    else:
        revisionToUse, leakConfigName, resultDirName, numMinsToSleep, isInstrumented = restArgs

    startup(revisionToUse, leakConfigName, resultDirName, numMinsToSleep, isInstrumented)
