
"""Descreption: This file provides some general functions to use in refactoring .sh scripts to .py scripts for running
aging-related experiments.
"""


import sys
import time
import os
import shutil
import glob
import subprocess
import re
import zipfile


def get_cmd_arg(i):
    """retrieve the passed argument via cmd line
        @param i: the number of needed argument passed via cmd line
    """
    listCmdArgs = sys.argv
    return listCmdArgs[i]


def total_cmd_args():
    """retrieve the total number of passed arguments
    """
    totalCmdArgs = len(sys.argv)
    return totalCmdArgs


def current_time():
    """retrieve the current time
    """
    now = time.strftime('%Y-%m-%d %H:%M:%S')
    return now


def sleep(minute):
    """puts current thread into sleep for *minute* minutes
    @param minute: take an int
    """
    x = float(minute)
    sleepMin = time.sleep(x * 60)
    return sleepMin


def join_paths(basePath, directoryName):
    """create a path based on a base path and the directory name
    """
    path = os.path.join(basePath, directoryName)
    return path


def read_files_to_list(path):
    """read all files from a specific path into a list of strings
    """
    f = []
    for inputName in glob.glob(os.path.join(path, '*.log')):
        if inputName.endswith(".log"):
            f.append(inputName)
    return f


def file_size(filename):
    """retrieve size of a given file
    """
    size = os.path.getsize(filename)
    return size


def move_file(source, destination):
    """move a file to another directory and overwrite it if existed and remove the source file
    """
    shutil.copy(source, destination)
    os.remove(source)
    return


def move_multi_files(source, destination, extension=None):
    """move files with specific extensions to another directory
    and overwrite it if existed and remove the source files
    """
    if extension:
        extensionList = [extension]
    else:
        extensionList = ['*.csv', '*.log', '*.prof']
    for item in extensionList:
        for filename in glob.glob(os.path.join(source, '%s') % item):
            shutil.copy(filename, destination)
            os.remove(filename)
    return


def make_dir(path):
    """create a directory based on the given path
    """
    if not os.path.exists(path):
        os.makedirs(path)
        os.chmod(path, 0777)
    return


def compute_total_rss(input_tuple):
    """ retrieve total rss amount
    @param input_tuple: the tuple of lines with rss amount
    """
    totalRss = 0
    x = input_tuple[0]
    text = x.splitlines()
    for item in text:
        for s in item.split():
            if s.isdigit():
                rss = int(s)
                totalRss += rss
    print totalRss
    return totalRss


def createLogFile(logPath, revisionToUse, leakConfigName, iteration):
    """create log name
    @param logPath: path of the log file
    @param revisionToUse: hadoop used revision
    @param leakConfigName: leak config name
    @param iteration: k-means iteration
    """
    logFileName = 'v' + revisionToUse + '-' + leakConfigName + '-' + iteration + '.log'
    logFile = logPath + '/' + logFileName
    return logFile


def dependencies(currentDep, revisionToUseDir, libPath):
    """add dependencies to run java command for unit test experiment
    @param currentDep:
    @param revisionToUseDir: hadoop used revision
    @param libPath: path of the dependency library defined in the Constans file
    """
    libName = revisionToUseDir + libPath
    for dep in os.listdir(libName):
        if '.jar' in dep:
            currentDep += ':' + libName + dep
    return currentDep


def getLeakValue(section, listLeakValue):
    """retrieve the leak values for different leaks
    :param section: section name of the config file
    :param listLeakValue: list of value that a leak can take it
    """
    digit = filter(str.isdigit, section)
    split = list(digit)
    if digit.find('0') == -1:
        for item in split:
            if item == '1':
                listLeakValue[0] = '2'
            elif item == '2':
                listLeakValue[1] = '2'
            elif item == '3':
                listLeakValue[2] = '2'
            elif item == '4':
                listLeakValue[3] = '2'
            elif item == '5':
                listLeakValue[4] = '2'
    else:
        countZero = digit.count('0')
        for item in split:
            if re.match('[1-9]', item):
                listLeakValue[int(item)-1] = '1' + countZero * '0'
    return listLeakValue


def retrieveUnitTests(jarFilePath):
    """retrieve unit tests from jar file"""
    listUnitTests = []
    listTestFiles = zipfile.ZipFile(jarFilePath).namelist()
    for testFile in listTestFiles:
        if os.path.basename(testFile).startswith('Test') and '$' not in testFile and testFile.endswith('.class'):
            testName = os.path.splitext(testFile)[0].replace('/', '.')
            listUnitTests.append(testName)
    return listUnitTests


def run_process(processName, listCmdArgs, workingDir=None, log=None):
    """Runs a command via python "call" function, waits to complete,
    write to the log file and returns return-code attribute
    @param processName: name of the command to run (possibly with path)
    @param listCmdArgs: list of command line arguments
    @param workingDir: if not None, changes first to this working dir (CAUTION: does NOT restore the working dir)
    """
    if workingDir:
        os.chdir(workingDir)

    # add processName and listCmdArgs to a list
    cmd = []
    cmd += [processName]
    cmd = cmd + listCmdArgs
    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    # print the output to the log file
    if log:
        logfile = log
        f = open(logfile, 'a')
        for line in process.stdout.readlines():
            f.write(line)
            print line.rstrip()
    else:
        sys.stdout.flush()
    process.wait()
    return process.returncode
